%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Syfte:    Lser, med cG(1)-metoden, differentialekvationen%
%%
%%          -(a(x)*u'(x))' + c(x)*u(x) = f(x),    x_min < x < x_max,%
%%
%%           med Robinrandvillkor%
%%
%%   a(x_min)*u'(x_min) = gamma(x_min)*(u(x_min) - g_D(x_min)) + g_N(x_min)
%%   -a(x_max)*u'(x_max) = gamma(x_max)*(u(x_max) - g_D(x_max)) + g_N(x_max)
%%
%%           och "plottar" lsningen.%
%%
%% Variabler:
%% p:  mesh           (Vektor med dimension 1 x N)
%% A:  styvhetsmatris (Matris med dimension N x N)
%% Mc:  massmatris    (Matris med dimension N x N)
%% R:  randmatris     (Matris med dimension N x N)
%% b:  lastvektor     (Vektor med dimension N x 1)
%% rv:  randvektor    (Vektor med dimension N x 1)
%% U:  lsning        (Vektor med dimension N x 1)
%%
%% Funktioner: a.m  definierar funktionen       a(x), x_min < x < x_max
%%             c.m  definierar funktionen       c(x), x_min < x < x_max
%%             f.m  definierar funktionen       f(x), x_min < x < x_max
%%
%%             gamma.m  definierar funktionen   gamma(x),   x = x_min, x_max
%%             g_D.m  definierar funktionen     g_D(x),   x = x_min, x_max
%%             g_N.m  definierar funktionen     g_N(x),   x = x_min, x_max

%%
%% MyFirstPoissonAssembler.m  berknar A, Mc, R, b, och rv
%%
%%

%% Exempel:  Du vill lsa ekvationen
%%             -u''(x) + x*u(x) = (1 + x)*sin(x),
%%             med randvillkoren
%%             u(0) = 0,
%%             u'(pi/2) = 0,
%%             p en likformig indelning av [0, pi/2] i 9 delintervall.

%%             Brja med att definiera funktionerna a(x) = 1, c(x) = x,
%%             och f(x) = (1 + x)*sin(x), i filerna a.m, c.m, och f.m.

%%             Drefter definierar du funktionerna gamma, g_D, och g_N,
%%             i filerna gamma.m, g_D.m, och g_N.m.

%%             Tips: Dessa funktioner, som definierar randvillkoren, r endast
%%             definierade i randpunkterna x = x_min och x = x_max. 
%%             Inargumentet fr allts bara anta dessa tv vrden, och
%%             funktionerna skall returnera motsvarande funktionsvrde.
%%             Funktionen gamma skulle drfr kunna tnkas konstrueras
%%             p fljande stt:

%%             function y = gamma(x)
%%             x_min = 0;
%%             x_max = pi/2;
%%             if x == x_min
%%              y = 10000; % Dirichletrandvillkor (approximativt)
%%             elseif x == x_max
%%              y = 0;     % Neumannrandvillkor
%%             else
%%              disp('Felaktigt argument i gamma.m')
%%             end

%% Det finns dock ett problem med konstruktionen ovan,
%% nmligen att det alltid r riskabelt att avgra ifall det rder
%% likhet mellan tv flyttal eftersom dessa inte representeras exakt i 
%% en dator. Ett annat (potentiellt) problem r att det skulle bli svrt att
%% generalisera rutinen till hgre dimension, dr randen r mer
%% komplicerad n bara tv punkter. Ett bttre stt (som ocks
%% enkelt generaliseras till hgre dimension) r att ge varje del
%% av randen (i detta fall varje punkt) ett id-nummer:

%% function y = gamma(rand_nr)
%%   if rand_nr == 1       % vi ger x_min nummer 1
%%     y = 10000;       % (approximativt) Dirichletrandvillkor
%%   elseif rand_nr == 2   % vi ger x_max nummer 2
%%     y = 0;           % Neumannrandvillkor
%%   else
%%     disp('Felaktigt argument i gamma.m')
%%   end

%% Definiera nu p samma stt g_D.m, och g_N.m, s att du fr de nskade
%% randvillkoren. (Kommentar: Eftersom gamma(x_max) = 0, spelar det ingen
%% roll vilket vrde du ger g_D(x_max).)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all        % Tar bl.a. bort alla variabler frn Matlab:s workspace.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             %
% Definiera meshen p.         %
%                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


p = 0:5/9:5; %Notera att 9 delintervall motsvarar N = 10 nodpunkter
                         %(inkl. randnoderna).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                    %
% Assemblera matriser och vektorer.  %
%                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


[A, Mc, R, b, rv] = MyFirstPoissonAssembler(p);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             %
% Ls ekvationssystemet.      %
%                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


VL = A + Mc + R;
hl = b + rv;

U = VL\hl;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                    %
% Plotta lsningen.                  %
%                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  


plot(p, U)
xlabel('x')
title('U(x)')




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                           % 
% I detta fall r den exakta lsningen (verifiera detta!)   %
%                                                           %
%     u(x) = u = -e^x + x + e^5 - 5                         %
%                                                           %
% s du kan berkna felet, u - U, enligt                    %
%                                                           %
% >>   u = -exp(p') + p' + exp(5) - 5;                      %
% >> fel = u - U;                                           %
%                                                           %
% (Vi transponerar 'p', eftersom 'U' r en kolonnvektor     %
%  och 'p' en radvektor.)                                   %
%                                                           %
% och plotta detta:                                         %
%                                                           %
% >> plot(p, fel)                                           %
%                                                           %
% Skriver du                                                %
%                                                           %
% >> max(abs(fel))                                          %
%                                                           %
% fr du en uppskattning av felet i max-norm                %
% (d.v.s. max ver alla noder).                             %
%                                                           %
% Det r mycket viktigt att testa sitt program genom att    %
% lsa ngra problem med knda (exakta) lsningar, och      %
% drefter jmfra den exakta med den berknade lsningen.  %
%                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
